/*-
 * Copyright (c) 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: devfs.c,v 1.3 2006/03/14 11:07:08 stas Exp $
 */

#include <stdlib.h>
#include <err.h>
#include <kvm.h>

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/user.h>
#define _KERNEL
#include <sys/mount.h>
#include <fs/devfs/devfs.h>
#undef _KERNEL

#include "fuser.h"

int
devfs_filestat(vp, fsp)
	const vnode_t	*vp;
	finfo_t		*fsp;
{
	struct devfs_dirent	devfs_dirent;
	struct mount		mount;
	struct vnode		vnode;
	int			ret;

	ASSERT(vp);
	ASSERT(fsp);

	ret = KVM_READ(kd, vp->v_data, &devfs_dirent, sizeof(devfs_dirent));
	if (ret != sizeof(devfs_dirent)) {
		warnx("can't read devfs_dirent at %p for vnode %p\n",
		    (void *)vp->v_data, vp);
		return 1;
	}

	ret = KVM_READ(kd, vp->v_mount, &mount, sizeof(mount));
	if (ret != sizeof(mount)) {
		warnx("can't read mount at %p for vnode %p\n",
		    (void *)vp->v_mount, vp);
		return 1;
	}

	ret = KVM_READ(kd, devfs_dirent.de_vnode, &vnode, sizeof(vnode));
	if (ret != sizeof(vnode)) {
		warnx("can't read vnode at %p for vnode %p\n",
		    (void *)devfs_dirent.de_vnode, vp);
		return 1;
	}

	fsp->dev = (long)mount.mnt_stat.f_fsid.val[0];
	fsp->ino = devfs_dirent.de_inode;

	return 0;
}
