/*-
 * Copyright (c) 2005 Stanislav Sedov <ssedov@mbsd.msk.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ntfs.c,v 1.2 2005/08/25 01:19:02 stas Exp $
 */

#include <stdlib.h>
#include <err.h>
#include <kvm.h>

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/mount.h>

#include <fs/ntfs/ntfs.h>
#include <fs/ntfs/ntfs_inode.h>

#include "fuser.h"

int
ntfs_filestat(vp, fsp)
	const vnode_t	*vp;
	finfo_t		*fsp;
{
	struct fnode		fnod;
	struct ntnode		node;
	int			ret;

	ASSERT(vp);
	ASSERT(fsp);

	ret = KVM_READ(kd, VTOF(vp), &fnod, sizeof(fnod));
	if (ret != sizeof(fnod)) {
		warnx("can't read ntfs fnode at %p\n", (void *)VTOF(vp));
		return 1;
	}

	ret = KVM_READ(kd, FTONT(&fnod), &node, sizeof(node));
	if (ret != sizeof(node)) {
		warnx("can't read ntfs node at %p\n", (void *)FTONT(&fnod));
		return 1;
	}

	fsp->ino = node.i_number;
	fsp->dev = dev2udev(node.i_dev);

	return 0;
}
