/*-
 * Copyright (c) 2005 Stanislav Sedov <ssedov@mbsd.msk.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: smbfs.c,v 1.1 2005/08/25 01:27:58 stas Exp $
 */

#include <stdlib.h>
#include <err.h>
#include <kvm.h>

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/vnode.h>
#define _KERNEL
#include <sys/mount.h>
#undef _KERNEL

#include <fs/smbfs/smbfs.h>
#include <fs/smbfs/smbfs_node.h>

#include "fuser.h"

int
smbfs_filestat(vp, fsp)
	const vnode_t	*vp;
	finfo_t		*fsp;
{
	struct smbnode	node;
	struct mount	mnt;
	int		ret;

	ASSERT(vp);
	ASSERT(fsp);

	ret = KVM_READ(kd, VTOSMB(vp), &node, sizeof(node));
	if (ret != sizeof(node)) {
		warnx("can't read smbfs fnode at %p\n", (void *)VTOSMB(vp));
		return 1;
	}

        ret = KVM_READ(kd, vp->v_mount, &mnt, sizeof(mnt));
        if (ret != sizeof(mnt)) {
                warnx("can't read mount at %p for vnode %p\n",
                    (void *)vp->v_mount, vp);
                return 1;
        }


	fsp->ino = node.n_ino;
	if (fsp->ino == 0)
		fsp->ino = 2;

	fsp->dev = mnt.mnt_stat.f_fsid.val[0];

	return 0;
}
